@@ -43,6 +43,8 @@
 #include <unistd.h>
 #endif
 
+#include <gio/gdesktopappinfo.h>
+
 #include <exo-helper/exo-helper.h>
 #include <exo-helper/exo-helper-utils.h>
 
@@ -51,6 +53,7 @@
 static void       exo_helper_finalize   (GObject        *object);
 static ExoHelper *exo_helper_new        (const gchar    *id,
                                          XfceRc         *rc);
+static void       clear_bad_entries     (XfceRc         *rc);
 
 
 
@@ -682,6 +685,23 @@ exo_helper_database_get_default (ExoHelperDatabase *database,
 
 
 
+static XfceRc*
+mimeapps_open (gboolean readonly)
+{
+  XfceRc *rc;
+
+  rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, "mimeapps.list", readonly);
+  if (G_UNLIKELY (rc == NULL))
+    {
+      /* deprecated location (glib < 2.41) */
+      rc = xfce_rc_config_open (XFCE_RESOURCE_DATA, "applications/mimeapps.list", readonly);
+    }
+
+  return rc;
+}
+
+
+
 /**
  * exo_helper_database_set_default:
  * @database : an #ExoHelperDatabase.
@@ -706,6 +726,7 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
   gchar       **mimetypes;
   guint         i;
   gchar        *path;
+  gchar        *entry;
 
   g_return_val_if_fail (category < EXO_HELPER_N_CATEGORIES, FALSE);
   g_return_val_if_fail (EXO_IS_HELPER_DATABASE (database), FALSE);
@@ -723,6 +744,11 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
   /* save the new setting */
   key = exo_helper_category_to_string (category);
   xfce_rc_write_entry (rc, key, exo_helper_get_id (helper));
+  g_free (key);
+
+  /* clear the dismissed preference */
+  key = g_strconcat (exo_helper_category_to_string (category), "Dismissed", NULL);
+  xfce_rc_delete_entry (rc, key, FALSE);
   xfce_rc_close (rc);
   g_free (key);
 
@@ -747,16 +773,11 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
     }
 
   /* open the mimeapp.list file to set the default handler of the mime type */
-  rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, "mimeapps.list", FALSE);
+  rc = mimeapps_open (FALSE);
   if (G_UNLIKELY (rc == NULL))
     {
-      /* deprecated location (glib < 2.41) */
-      rc = xfce_rc_config_open (XFCE_RESOURCE_DATA, "applications/mimeapps.list", FALSE);
-      if (G_UNLIKELY (rc == NULL))
-        {
-          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open %s for writing"), "mimeapps.list");
-          return FALSE;
-        }
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open %s for writing"), "mimeapps.list");
+      return FALSE;
     }
 
   /* open the exo desktop file to read the mimetypes the file supports */
@@ -770,17 +791,36 @@ exo_helper_database_set_default (ExoHelperDatabase *database,
       mimetypes = xfce_rc_read_list_entry (desktop_file, "X-XFCE-MimeType", ";");
       if (mimetypes != NULL)
         {
+          GDesktopAppInfo *info = g_desktop_app_info_new (filename);
+
           xfce_rc_set_group (rc, "Added Associations");
 
           for (i = 0; mimetypes[i] != NULL; i++)
             if (!exo_str_is_empty (mimetypes[i]))
-              xfce_rc_write_entry (rc, mimetypes[i], filename);
+              {
+                entry = g_strconcat (filename, ";", NULL);
+                xfce_rc_write_entry (rc, mimetypes[i], entry);
+                g_free (entry);
+
+                if (info != NULL)
+                  {
+                    g_app_info_set_as_default_for_type (G_APP_INFO (info),
+                                                        mimetypes[i],
+                                                        NULL);
+                  }
+              }
           g_strfreev (mimetypes);
+          if (info != NULL)
+            {
+              g_object_unref (info);
+            }
         }
 
       xfce_rc_close (desktop_file);
     }
 
+  clear_bad_entries (rc);
+
   xfce_rc_close (rc);
 
   return TRUE;
@@ -828,6 +868,11 @@ exo_helper_database_clear_default (ExoHelperDatabase *database,
   /* save the new setting */
   key = exo_helper_category_to_string (category);
   xfce_rc_delete_entry (rc, key, FALSE);
+  g_free (key);
+
+  /* clear the dismissed preference */
+  key = g_strconcat (exo_helper_category_to_string (category), "Dismissed", NULL);
+  xfce_rc_delete_entry (rc, key, FALSE);
   xfce_rc_close (rc);
   g_free (key);
 
@@ -852,16 +897,11 @@ exo_helper_database_clear_default (ExoHelperDatabase *database,
     }
 
   /* open the mimeapp.list file to set the default handler of the mime type */
-  rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, "mimeapps.list", FALSE);
+  rc = mimeapps_open (FALSE);
   if (G_UNLIKELY (rc == NULL))
     {
-      /* deprecated location (glib < 2.41) */
-      rc = xfce_rc_config_open (XFCE_RESOURCE_DATA, "applications/mimeapps.list", FALSE);
-      if (G_UNLIKELY (rc == NULL))
-        {
-          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open %s for writing"), "mimeapps.list");
-          return FALSE;
-        }
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_IO, _("Failed to open %s for writing"), "mimeapps.list");
+      return FALSE;
     }
 
   /* open the exo desktop file to read the mimetypes the file supports */
@@ -886,6 +926,8 @@ exo_helper_database_clear_default (ExoHelperDatabase *database,
       xfce_rc_close (desktop_file);
     }
 
+  clear_bad_entries (rc);
+
   xfce_rc_close (rc);
 
   return TRUE;
@@ -893,6 +935,62 @@ exo_helper_database_clear_default (ExoHelperDatabase *database,
 
 
 
+static void
+clear_bad_entry (XfceRc *rc,
+                 gchar  *key,
+                 gchar  *filename)
+{
+  gchar **values;
+
+  if (xfce_rc_has_entry (rc, key))
+    {
+      values = xfce_rc_read_list_entry (rc, key, ";");
+      if (values != NULL)
+        {
+          GSList *list = NULL, *item = NULL;
+          gint i;
+
+          for (i = 0; values[i] != NULL; i++)
+            {
+              if (!exo_str_is_empty(values[i]) && g_strcmp0(values[i], filename) != 0)
+                {
+                  list = g_slist_append (list, g_strdup(values[i]));
+                }
+            }
+          g_strfreev(values);
+
+          if (list == NULL)
+            {
+              xfce_rc_delete_entry (rc, key, FALSE);
+            }
+          else
+            {
+              gchar   *value;
+              GString *string = g_string_new (NULL);
+              for (item = list; item != NULL; item = g_slist_next (item))
+                {
+                  g_string_append_printf (string, "%s;", (gchar *)item->data);
+                }
+              value = g_string_free (string, FALSE);
+              xfce_rc_write_entry (rc, key, value);
+              g_slist_free_full (list, g_free);
+              g_free (value);
+            }
+        }
+    }
+}
+
+
+
+static void
+clear_bad_entries (XfceRc *rc)
+{
+  xfce_rc_set_group (rc, "Added Associations");
+  clear_bad_entry (rc, "x-scheme-handler/file", "exo-file-manager.desktop"); // Xfce #7257
+}
+
+
+
 static gint
 helper_compare (gconstpointer a,
                 gconstpointer b)
@@ -1087,3 +1185,75 @@ exo_helper_database_set_custom (ExoHelperDatabase *database,
   g_free (category_string);
   g_free (file);
 }
+
+/**
+ * exo_helper_database_get_dismissed:
+ * @database : an #ExoHelperDatabase.
+ * @category : an #ExoHelperCategory.
+ *
+ * Returns %TRUE if errors should no longer be displayed
+ * on the default #ExoHelper for the @category in @database.
+ *
+ * Return value: %TRUE if dismissed, %FALSE otherwise.
+ **/
+gboolean exo_helper_database_get_dismissed (ExoHelperDatabase *database,
+                                            ExoHelperCategory  category)
+{
+  XfceRc      *rc;
+  gchar       *key;
+  gboolean     dismissed = FALSE;
+
+  g_return_val_if_fail (EXO_IS_HELPER_DATABASE (database), FALSE);
+  g_return_val_if_fail (category < EXO_HELPER_N_CATEGORIES, FALSE);
+
+  rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, "xfce4/helpers.rc", TRUE);
+  if (G_LIKELY (rc != NULL))
+    {
+      key = g_strconcat (exo_helper_category_to_string (category), "Dismissed", NULL);
+      dismissed = xfce_rc_read_bool_entry (rc, key, FALSE);
+      xfce_rc_close (rc);
+      g_free (key);
+    }
+
+  return dismissed;
+}
+
+/**
+ * exo_helper_database_set_dismissed:
+ * @database  : an #ExoHelperDatabase.
+ * @category  : an #ExoHelperCategory.
+ * @dismissed : TRUE if the errr should no longer be displayed.
+ * @error     : return location for errors or %NULL.
+ *
+ * Dismisses future errors related to the selected helper category.
+ * This setting is cleared any time a new default is configured.
+ * Returns %TRUE on success, %FALSE if @error is set.
+ *
+ * Return value: %TRUE on success, %FALSE if @error is set.
+ **/
+gboolean
+exo_helper_database_set_dismissed (ExoHelperDatabase *database,
+                                   ExoHelperCategory  category,
+                                   gboolean           dismissed)
+{
+  XfceRc       *rc;
+  gchar        *key;
+
+  g_return_val_if_fail (category < EXO_HELPER_N_CATEGORIES, FALSE);
+  g_return_val_if_fail (EXO_IS_HELPER_DATABASE (database), FALSE);
+
+  /* open the helpers.rc for writing */
+  rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, "xfce4/helpers.rc", FALSE);
+  if (G_UNLIKELY (rc == NULL))
+    {
+      return FALSE;
+    }
+
+  /* save the new setting */
+  key = g_strconcat (exo_helper_category_to_string (category), "Dismissed", NULL);
+  xfce_rc_write_bool_entry (rc, key, dismissed);
+  xfce_rc_close (rc);
+  g_free (key);
+
+  return TRUE;
+}
